#!/usr/bin/env python
import os
import sys
import math
import getopt
from copy import copy
from base64 import encodestring, decodestring

def read_orders():
    res = {}
    for fname in os.listdir("./orders"):
        if not fname.endswith(".mdwn"):
            continue
        d = {}
        f = open(os.path.join("./orders", fname))
        for l in f.readlines():
            if l[0] != '*':
                break
            l = l[1:].strip().split(":", 1)
            if len(l) <= 1:
                break
            d[l[0].lower().strip()] = l[1].strip()
        if d:
            res[fname] = d
        f.close()
    return res

def apply_filters(orders, filters):
    res = {}
    f = map(lambda x: filter(x, orders), filters)
    for k,v in f[0]:
        res[k] = v
    return res

def stable_filter(order):
    return order[1]['stage'].lower().find('stable') >= 0

def beta_filter(order):
    return order[1]['stage'].lower().find('beta') >= 0

def alpha_filter(order):
    if not order[1].has_key('stage'):
        print order
    return order[1]['stage'].lower().find('alpha') >= 0

def print_orders(orders):
    for fname, order in orders.items():
        print fname
        for k, v in order.items():
            print "\t\t%s: %s" % (k, v)

def get_max_unit_cost(fname, order):
    price = order.get("max unit price", None)
    if not price:
        price = "10000000.0"
    if price[0] == "$":
        price = price[1:]
    price = price.strip()
    try:
        return float(price)
    except:
        print fname, order
        sys.exit(0)

def get_quantity_range(fname, order):
    qty = order.get("quantity")
    qty = qty.split(" ")[0]
    qty = qty.split("-")
    try:
        qty = map(int, qty)
    except:
        print fname, order
        sys.exit(0)
    if len(qty) == 1:
        return (qty[0], qty[0])
    return qty

def get_max_budget(fname, order):
    budget = order.get("max budget", None)
    if not budget:
        budget = "10000000.0"
    if budget[0] == "$":
        budget = budget[1:]
    budget = budget.strip()
    try:
        return float(budget)
    except:
        print fname, order
        sys.exit(0)

def count_orders(orders, unit_cost):
    res = {}
    count = 0
    for (fname, order) in orders.items():
        budget = get_max_budget(fname, order)
        price = get_max_unit_cost(fname, order)
        min_qty, max_qty = get_quantity_range(fname, order)
        if unit_cost > price:
            continue
        actual_qty = math.floor(budget / unit_cost)
        actual_qty = min(actual_qty, max_qty)
        #print fname, unit_cost, budget, price, min_qty, max_qty, actual_qty
        if actual_qty < min_qty:
            continue
        res[fname] = order
        count += actual_qty
    return res, count

# stage, min_quantity, max_quantity, NREs, unit_cost
estimated_pricing = [\
("alpha", 10, 400, 0.0, 25.0),
("alpha", 10, 400, 0.0, 30.0),
("alpha", 10, 400, 0.0, 35.0),
("alpha", 10, 400, 0.0, 40.0),
("alpha", 10, 400, 0.0, 50.0),
("alpha", 10, 400, 0.0, 60.0),
("alpha", 10, 400, 0.0, 80.0),
("alpha", 10, 400, 2000.0, 30.0),
("alpha", 10, 100, 2000.0, 50.0),
("alpha", 10, 100, 2000.0, 80.0),
("beta", 10, 99, 0.0, 80.0),
("beta", 100, 999, 0.0, 100.0),
("beta", 100, 999, 0.0, 80.0),
("beta", 100, 999, 0.0, 60.0),
("beta", 100, 999, 0.0, 50.0),
("beta", 100, 999, 0.0, 25.0),
("stable", 100, 999, 0.0, 100.0),
("stable", 100, 999, 0.0, 80.0),
("stable", 100, 999, 0.0, 60.0),
("stable", 1000, 9999, 0.0, 100.0),
("stable", 1000, 9999, 0.0, 80.0),
("stable", 1000, 9999, 0.0, 50.0),
("stable", 1000, 9999, 0.0, 35.0),
("stable", 10000, 99999, 0.0, 30.0),
("stable", 100000, 1000000, 0.0, 25.0),
("stable", 100000, 1000000, 0.0, 20.0),
]

estimated_cost = 35.0

def rot(x, rotate):
    s = "abcdefghijklmnopqrstuvwxyz"
    r = ''
    for c in x:
        if c in s:
            idx = s.index(c)
            idx = (idx + rotate) % 26
            c = s[idx]
        r += c
    return r
    
def translate_email(**kwargs):

    orders = kwargs['orders']
    for (k, v) in orders.items():
        c = v['contact']
        pc = v['contact'].lower()
        if pc.startswith("rot13"):
            pc = rot(pc[5:], 13)
        if pc.startswith("%s/"):
            pc = pc[3:].split(' ')
            rep = pc[0].split("/")
            pc = ' '.join(pc[1:])
            pc = pc.replace(rep[0], rep[1])
        if pc.startswith("`(") and pc.endswith(")`"):
            pc = pc[2:-2].split(' ')
            pc = map(int, pc)
            pc = ''.join(map(chr, pc))
        pc = pc.replace("//-- --//", "@")
        pc = pc.replace("deleteme", "")
        pc = pc.replace("_goawayrobot_", "")
        pc = pc.replace("blabla", "")
        pc = pc.replace("first name", "")
        pc = pc.replace("last name", "")
        pc = pc.replace("removethis", "")
        pc = pc.replace("remove", "")
        pc = pc.replace("(remove this ok?)", "")

        pc = pc.replace("underscore", "_")
        pc = pc.replace("(dash)", "-")
        pc = pc.replace("(plus)", "+")

        pc = pc.replace("(a7)", "@")
        pc = pc.replace('"at"', "@")
        pc = pc.replace("(-at-)", "@")
        pc = pc.replace("( at )", "@")
        pc = pc.replace(" ayt ", "@")
        pc = pc.replace(" add ", "@")
        pc = pc.replace("(at .)", "@")
        pc = pc.replace(" a t ", "@")
        pc = pc.replace(" a-t ", "@")
        pc = pc.replace("(at symbol)", "@")
        pc = pc.replace("[at (@)]", "@")
        pc = pc.replace("[a:t]", "@")
        pc = pc.replace("#{@}", "@")
        pc = pc.replace("(@)", "@")
        pc = pc.replace("{(at)}", "@")
        pc = pc.replace("{+at+}", "@")
        pc = pc.replace("[@]", "@")
        pc = pc.replace("[@!]", "@")
        pc = pc.replace(" at ", "@")
        pc = pc.replace("-at-", "@")
        pc = pc.replace("_at_", "@")
        pc = pc.replace("_a_t_", "@")
        pc = pc.replace("_at_", "@")
        pc = pc.replace("a-t", "@")
        pc = pc.replace(" at ", "@")
        pc = pc.replace(" in ", "@")
        pc = pc.replace(" att ", "@")
        pc = pc.replace(" (a t) ", "@")
        pc = pc.replace(" [att] ", "@")
        pc = pc.replace(" [curly-a] ", "@")
        pc = pc.replace("(at the rate)", "@")
        pc = pc.replace(" arobase ", "@")
        pc = pc.replace("//at//", "@")
        pc = pc.replace("/at/", "@")
        pc = pc.replace("#at#", "@")
        pc = pc.replace("(at)", "@")
        pc = pc.replace("[at ]", "@")
        pc = pc.replace("[at)", "@")
        pc = pc.replace("[]a()t", "@")
        pc = pc.replace("{at}", "@")
        pc = pc.replace("<at>", "@")
        pc = pc.replace("{ta}", "@")
        pc = pc.replace(" {a} ", "@")
        pc = pc.replace(" (a) ", "@")
        pc = pc.replace("<*>", "@")
        pc = pc.replace("[at@]", "@")
        pc = pc.replace("[@t]", "@")
        pc = pc.replace(" <-localp-domainp-> ", "@")
        pc = pc.replace(" [aaaaaattttt] ", "@")
        pc = pc.replace("---at---", "@")
        pc = pc.replace("--at--", "@")
        pc = pc.replace(" (:a.t:) ", "@")
        pc = pc.replace("-at-", "@")
        pc = pc.replace(" ding ", "@")
        pc = pc.replace(" guess ", "@")
        pc = pc.replace("<-@+->", "@")
        pc = pc.replace(" #a[y]# ", "@")
        pc = pc.replace("(a)", "@")
        pc = pc.replace("atthing", "@")
        pc = pc.replace("<aaat>", "@")
        pc = pc.replace("< a t >", "@")
        pc = pc.replace("[at]", "@")
        pc = pc.replace("[att]", "@")
        pc = pc.replace(" a-t ", "@")
        pc = pc.replace("(at)", "@")
        pc = pc.replace("{@}", "@")
        pc = pc.replace("(youknow)", "@")
        pc = pc.replace(" [a] ", "@")
        pc = pc.replace("(-@-)", "@")
        pc = pc.replace("[]gmail.com", "@gmail.com")
        pc = pc.replace("# gmail.com", "@gmail.com")
        pc = pc.replace("#{.}", ".")
        pc = pc.replace("{+dot+}", ".")
        pc = pc.replace("/dot/", ".")
        pc = pc.replace("_ dot _", ".")
        pc = pc.replace('"dot"', ".")
        pc = pc.replace("_dot_", ".")
        pc = pc.replace(" dott ", ".")
        pc = pc.replace(" dottu ", ".")
        pc = pc.replace(" d-o-t ", ".")
        pc = pc.replace("_do_t_", ".")
        pc = pc.replace("(dot)", ".")
        pc = pc.replace(" doot ", ".")
        pc = pc.replace("(period)", ".")
        pc = pc.replace("-period-", ".")
        pc = pc.replace("(d0t)", ".")
        pc = pc.replace("(d)", ".")
        pc = pc.replace("(d0T)", ".")
        pc = pc.replace("(d0t)", ".")
        pc = pc.replace("{dot}", ".")
        pc = pc.replace("<dot>", ".")
        pc = pc.replace("{tod}", ".")
        pc = pc.replace("{dot}", ".")
        pc = pc.replace("{.}", ".")
        pc = pc.replace("(dot)", ".")
        pc = pc.replace("[d:o:t]", ".")
        pc = pc.replace("< d o t >", ".")
        pc = pc.replace("{+}", ".")
        pc = pc.replace("#dot#", ".")
        pc = pc.replace("[dot]", ".")
        pc = pc.replace("[dott]", ".")
        pc = pc.replace("[dot (.)]", ".")
        pc = pc.replace("(.)", ".")
        pc = pc.replace("[dot(.)]", ".")
        pc = pc.replace("/ dot |", ".")
        pc = pc.replace(" dot ", ".")
        pc = pc.replace(" dawt ", ".")
        pc = pc.replace("~.~", ".")
        pc = pc.replace(" do-t ", ".")
        pc = pc.replace(" do .t ", ".")
        pc = pc.replace("_dot_", ".")
        pc = pc.replace("_dot_", ".")
        pc = pc.replace("--dot--", ".")
        pc = pc.replace("---dot---", ".")
        pc = pc.replace("-dot-", ".")
        pc = pc.replace("(_._)", ".")
        pc = pc.replace("{{.}}", ".")
        pc = pc.replace("(this-is-a-d0t)", ".")
        pc = pc.replace("(as-is-this)", ".")
        pc = pc.replace(" dooot ", ".")
        pc = pc.replace(" d o t ", ".")
        pc = pc.replace(" doot_doot ", ".")
        pc = pc.replace(" [dottttt] ", ".")
        pc = pc.replace(" dot ", ".")
        pc = pc.replace(" d0t ", ".")
        pc = pc.replace(" d0t ", ".")
        pc = pc.replace("(^dot^)", ".")


        pc = pc.replace(" has his email from google.", "@gmail.com")
        pc = pc.replace("@@", "@")
        pc = pc.replace("(twenty one)", "21")
        pc = pc.replace("hot-mail (without the - )", "hotmail")
        pc = pc.replace("-fakebox-", "")
        pc = pc.replace("#yahoo", "@yahoo")
        pc = pc.replace("yahoo com", "yahoo.com")
        pc = pc.replace("yaahoo", "yahoo")
        pc = pc.replace("gmail com", "gmail.com")
        pc = pc.replace("g-mail.com", "gmail.com")
        pc = pc.replace(",com", ".com")
        pc = pc.replace(" c0m ", "com")
        pc = pc.replace(" ", "")
        pc = pc.replace("(removethis)", "")
        pc = pc.replace("(removethis)", "")
        pc = pc.replace("h0tmail", "hotmail")
        pc = pc.replace("thegoogleemailservice", "gmail.com")
        pc = pc.replace("geemayill.k0m", "gmail.com")
        pc = pc.replace("g00gle.c0m", "google.com")
        pc = pc.replace("google'semailservice", "gmail.com")
        pc = pc.replace("google'smail", "gmail.com")
        pc = pc.lower()
        if pc.endswith("@gmail"):
            pc += ".com"
        if pc.endswith("@yahoo"):
            pc += ".com"
        pc = pc.replace("/", "")
        pc = pc.replace("\\", "")
        pc = pc.replace("<", "")
        pc = pc.replace("{", "")
        pc = pc.replace("(", "")
        pc = pc.replace("[", "")
        pc = pc.replace(">", "")
        pc = pc.replace("}", "")
        pc = pc.replace(")", "")
        pc = pc.replace("]", "")

        if "." in pc and "@" in pc:
            v['email'] = pc

def report_1stbatch(**kwargs):

    total_quantities = 0

    for (k, v) in orders.items():
        if report == 'confirmed':
            if v['status'].lower().find('1stbatch') < 0:
                continue
        else:
            if not v.has_key('email'):
                continue
            if v['status'].lower().find('1stbatch') < 0:
                if v['stage'].lower().find('alpha') < 0:
                    continue
        # work out costing
        (min_qty, max_qty) = get_quantity_range(k, v)
        budget =  get_max_budget(k, v)
        max_unit_cost = get_max_unit_cost(k, v)
        actual_qty = math.floor(budget / max_unit_cost)
        actual_qty = min(actual_qty, max_qty)
        if actual_qty < min_qty:
            continue
        if max_unit_cost < target_unit_cost:
            continue
        if report == 'email':
            print v['email']
        else:
            print "%-25s %-35s %d @ $%5.2f" % \
                    (k[:25][:-5], v['email'][:35], actual_qty, max_unit_cost)
        total_quantities += actual_qty

    if report == 'email':
        return
    else:
        print "total numbers @ $%5.2f : %d" % \
                    (target_unit_cost, total_quantities)


def report_email(**kwargs):

    for (k, v) in orders.items():
        if v.has_key('email'):
            print "%-39s %-39s" % (v['email'], v['contact'])

    for (k, v) in orders.items():
        if not v.has_key('email'):
            print "!%-39s\t%-39s" % (v['contact'], k)

def report_overview(**kwargs):

    orders = kwargs['orders']

    for (stage, min_q, max_q, target_nres, target_unit_cost) in estimated_pricing:
        forders = copy(orders)
        forders, total_orders, current_unit_cost = calc_quantities(**locals())
        profit = (current_unit_cost - estimated_cost) * total_orders
        print stage, min_q, max_q, target_nres, target_unit_cost, total_orders, current_unit_cost, total_orders*current_unit_cost, profit
        

def calc_quantities(**kwargs):

    orders = kwargs['orders']
    stage = kwargs['stage']
    if stage == 'alpha':
        ft = alpha_filter
    elif stage == 'beta':
        ft = beta_filter
    elif stage == 'stable':
        ft = stable_filter
    forders = apply_filters(orders.items(), [ft])

    target_nres = kwargs['target_nres']
    target_unit_cost = kwargs['target_unit_cost']

    # ok this is a bit messed up: start off at zero cost (nearly), 
    # removing people who fall out of the criteria.  from that number
    # of people, work out the unit cost, check that's ok, repeat until you
    # have only the people left who match the criteria.  as the unit cost
    # increases with the number of people who fall out, you can stop when
    # no more people are removed.

    step = 0.01
    percentage = step
    unit_cost = 0.001
    total_costs = 0.0
    current_unit_cost = 0.001

    odd1 = False

    unit_cost = target_unit_cost
    total_costs = target_nres 
    forders, total_orders = count_orders(forders, current_unit_cost)
    order_len = len(forders)
    old_len = -1

    while old_len != order_len:
        current_unit_cost = unit_cost + (total_costs / total_orders)
        forders, total_orders = count_orders(forders, current_unit_cost)
        old_len = order_len
        order_len = len(forders)

    return forders, total_orders, current_unit_cost

def report_quantities(**kwargs):

    alpha, total_orders, current_unit_cost = calc_quantities(**kwargs)
    print_orders(alpha)
    print "total orders:", total_orders, "unit cost:", current_unit_cost

if __name__ == '__main__':

    html_mode = False
    mode = 'quantities'
    report = ''
    stage = 'alpha'
    target_nres = 2000.0
    target_unit_cost = 30.0

    opts, args = getopt.getopt(sys.argv[1:], "m:h:s:p:r:")
    for (opt, arg) in opts:
        if opt == '-h':
            html_mode = True
        if opt == '-m':
            mode = arg
        if opt == '-p':
            target_unit_cost = float(arg)
        if opt == '-r':
            report = arg
        if opt == '-s':
            stage = arg

    orders = read_orders()

    if mode == 'quantities':
        report_quantities(**locals())
    elif mode == 'overview':
        report_overview(**locals())
    elif mode == 'emailtranslate':
        translate_email(**locals())
        report_email(**locals())
    elif mode == '1stbatch':
        translate_email(**locals())
        report_1stbatch(**locals())

